package dbapp;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Savepoint;
import java.text.MessageFormat;
import java.util.Locale;
import java.util.Locale.Category;
import java.util.ResourceBundle;

public class Main {

	public static void main(String[] args) throws SQLException, IOException {
		try {
			Class.forName("org.hsqldb.jdbcDriver");
		} catch (Exception e) {
			System.out.println("ERROR: failed to load HSQLDB JDBC driver.");
			e.printStackTrace();
			return;
		}

		Connection c = DriverManager.getConnection("jdbc:hsqldb:hsql://localhost/xdb", "sa", "");
		c.setAutoCommit(false);

		BufferedReader in = new BufferedReader(new InputStreamReader(System.in));

		Locale currLocale = Locale.getDefault();
		ResourceBundle messages = ResourceBundle.getBundle("ApplicationMessages", currLocale);
		MessageFormat rowFormat = new MessageFormat("{0}: {1,number,currency}", currLocale);

		c.createStatement().execute("CREATE TABLE IF NOT EXISTS Items (ItemName varchar(255), Price float);");
		c.commit();

		Savepoint save = null;

		mainLoop: while (true) {
			String command = in.readLine();
			String[] commandParts = command.split(" ");
			switch (commandParts[0]) {
			case "modify":
				save = c.setSavepoint();
				break;
			case "lang":
				currLocale = Locale.forLanguageTag(commandParts[1]);
				messages = ResourceBundle.getBundle("ApplicationMessages", currLocale, Main.class.getClassLoader());
				rowFormat = new MessageFormat("{0}: {1,number,currency}", currLocale);
				break;
			case "done":
				c.commit();
				save = null;
				break;
			case "exit":
				break mainLoop;
			case "abort":
				c.rollback(save);
				save = null;
				break;
			case "clear":
				if (save != null) {
					c.createStatement().execute("DELETE FROM Items");
				} else {
					System.err.println(messages.getString("ClearNotInModifyMode"));
				}
				break;
			case "add":
				if (save != null) {
					PreparedStatement stmt = c.prepareStatement("INSERT INTO Items VALUES (?, ?)");
					stmt.setString(1, commandParts[1]);
					stmt.setFloat(2, Float.parseFloat(commandParts[2]));
					stmt.executeUpdate();
				} else {
					System.err.println(messages.getString("AddNotInModifyMode"));
				}
				break;
			case "view":
				ResultSet items = c.createStatement().executeQuery("SELECT ItemName, Price FROM Items");
				while (items.next()) {
					System.out.println(rowFormat.format(new Object[] { items.getString(1), items.getFloat(2) }));
				}
				break;

			default:
				System.err.println("Unknown command: " + command);
				break;
			}
		}

		c.close();
	}

}
